START TRANSACTION;
DROP TABLE IF EXISTS airspace CASCADE;
DROP TABLE IF EXISTS airfields CASCADE;
DROP TABLE IF EXISTS countries CASCADE;
DROP SEQUENCE IF EXISTS airfields_id_seq CASCADE;
DROP SEQUENCE IF EXISTS airspace_id_seq CASCADE;
DROP SEQUENCE IF EXISTS countries_id_seq CASCADE;

CREATE SEQUENCE airfields_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

CREATE SEQUENCE airspace_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

CREATE SEQUENCE countries_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;


CREATE TABLE airspace
(
  id integer NOT NULL DEFAULT nextval('airspace_id_seq'::regclass),
  name character varying(256),
  geometry geometry,
  type character varying(32),
  class character varying(32),
  region character varying(4),
  description text,
  alt_min character varying(64),
  alt_max character varying(64),
  update timestamp with time zone DEFAULT now(),
  CONSTRAINT airspace_pkey1 PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

CREATE TABLE airfields
(
  id integer NOT NULL DEFAULT nextval('airfields_id_seq'::regclass),
  region character varying(4),
  name character varying(256),
  code character varying(256),
  description text,
  elevation integer,
  direction integer,
  frequency numeric,
  sym character varying(256),
  update timestamp with time zone DEFAULT now(),
  length integer,
  "position" geometry,
  CONSTRAINT points_pkey PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

CREATE TABLE countries (
   id integer NOT NULL DEFAULT nextval('countries_id_seq'::regclass),
  short_name character varying(4),
  name character varying(256),
  lat_min numeric,
  lon_min numeric,
  lat_max numeric,
  lon_max numeric
);

COMMIT;