<?php

//echo "disabled"; exit();
error_reporting(E_ALL & ~E_NOTICE);

require_once('config.php');
require_once('../geo.php');

$dbconn = pg_connect($config['dbconn']) or die('Could not connect: ' . pg_last_error());


$files = array(
  'CZ' => 'CZ_combined_2014_05_01.txt',
//  'SK' => 'SK2012.txt',
//  'DE' => 'Germany_CW14_2013.txt',
//  'AT' => 'austria_ATS_2011_openair.txt',
//  'PL' => '18_SEP_2014_Polska.txt',

);

$files_nicenames = array (
  'CZ' => 'Česká republika',
  'SK' => 'Slovensko',
  'AT' => 'Rakousko',
  'PL' => 'Polsko',
  'DE' => 'Německo',
);


function getAirspaceType($AN, $AC) {

  $patterns = array(
    '/FIR/' => 'FIR',
    '/MCTR/' => 'MCTR',
    '/CTR/' => 'CTR',
    '/CTA/' => 'CTA',
    '/^LK D/' => 'D',
    '/^LK P/' => 'P',
    '/^LK R/' => 'R',
    '/^LK TRA/' => 'TRA',
    '/^LK TSA/' => 'TSA',
    '/^TMA/' => 'TMA',
    '/^MTMA/' => 'MTMA',

    '/^LZD/' => 'D',
    '/^LZP/' => 'P',
    '/^LZR/' => 'R',
    '/^LZTRA/' => 'TRA',
    '/^LZTSA/' => 'TSA',
    '/^TSA/' => 'TSA',
    '/AREA.C/' => 'C',
    '/^EP TSA/' => 'TSA',
    '/^ED.*R/' => 'R',
    '/^EP.*R/' => 'R',
    '/^EP.*P/' => 'P',
    '/TMZ-/' => 'TMA',
//    '/FIS-GDA/' => 'FIS-GDA',
  );

  foreach ($patterns as $regexp => $target) {
    if (preg_match($regexp, $AN)) {
      return $target;
    }
  }
  echo "cannot identify type according to name: <b>".$AN . ' '. $AC. " </b><br/>\n";
  return "CLASS".$AC;
}


function DA_V($DA, $V) {
  return array("DA_V ($DA, $V)");
}

function DB_V($DB, $V) {

  $coord = explode(",", $DB);
  $C1 = DMSToDecimal(trim($coord[0]));
  $C2 = DMSToDecimal(trim($coord[1]));
  $CX = DMSToDecimal($V['X']);

  $D = (isset($V['D']) && ($V['D'] == "+"));

  return insertArcII($CX, $C1, $C2, $D);
}


function DC_V($DC, $V) {
  $r = (float)$DC;
  $pos = DMSToDecimal($V);
  $poly = insertCircle($pos, $r);
  return $poly;
}

function remove_comments($str) {
  return trim(explode("*", $str, 2)[0]);
}

function AC_to_color($ac) {

  $colors = array (
    'R'   => 'Blue',
    'Q'   => 'DarkBlue',
    'P'   => 'DarkRed',
    'A'   => 'Red',
    'B'   => 'DarkGray',
    'C'   => "#00ffff",
    'D'   => "#660000",
    'GP'  => 'LightGray',
    'CTR' => 'Green',
    'W'   => "#660066",
  );

  if (isset($colors[$ac])) {
    return $colors[$ac];
  }
  return "black";
}


function DMSToDecimal($str) { // 49:50:18 N 015:37:24 E -> 49.838333333333,15.623333333333
  $str = strtoupper($str);
  if (preg_match("/(\d+):(\d+\.?\d*):?(\d+\.?\d*)\s*([N|S])\s*(\d+):(\d+\.?\d*):?(\d+\.?\d*)\s*([E|W])/", $str, $a)) {
    $lat = (($a[4] == "N") ? 1 : -1) * (floor($a[1]) + floor($a[2])/60.0 + $a[3]/3600.0);
    $lon = (($a[8] == "E") ? 1 : -1) * (floor($a[5]) + floor($a[6])/60.0 + $a[7]/3600.0);
  } else {
    die ("error in function DMSToDecimal($str)");
  }
  return array('lat' => $lat, 'lon' => $lon);
}


function parseOpensoaring($filename) {
  if (!file_exists($filename)) {
    throw new Exception("file not exists ".$filename);
  }
$file_contents = file_get_contents($filename);
$lines = explode( "\n", $file_contents);

$ao = array();

$all_ao = array();

foreach ($lines as $line) {
  $line = remove_comments($line);
  $parts = explode(" ", $line, 2);
  switch ($parts[0]) {
    case 'AC': // Airspace Class
      if (isset($ao['AC'])) {
        array_push($all_ao, $ao);
        $ao = array();
      }
      $ao['AC'] = $parts[1];
      $ao['DP'] = array();
    break;
    case 'AN': // Airspace Name
      $ao['AN'] = $parts[1];
    break;
    case 'AH': // Airspace Ceiling
      $ao['AH'] = $parts[1];
    break;
    case 'AL': // Airspace Floor
      $ao['AL'] = $parts[1];
    break;
    case 'AT': // Coordinate of where to place a name label on the map (optional)
      $ao['AT'] = DMSToDecimal($parts[1]);
    break;
    case 'TO': // Declares Terrain Open Polygon; string = name (optional)
      die("TO is unsupported");
    break;
    case 'TC': // Declares Terrain Closed Polygon; string = name (optional)
      die("TC is unsupported");
    break;
    case 'SP': // Selects Pen to be used in drawing
      $ao['SP'] = $parts[1];
    break;
    case 'SB': // Selects Brush to be used in drawing
      $ao['SB'] = $parts[1];
    break;
    case 'DP': // add polygon pointC
       array_push($ao['DP'], DMSToDecimal($parts[1]));
    break;
    case 'DA': // DA radius, angleStart, angleEnd    ; add an arc, angles in degrees, radius in nm (set center using V X=...)
      $ao['DA'] = $parts[1];
      if (isset($ao['V'])) {
         $ao['DP'] = array_merge($ao['DP'], DA_V($ao['DA'], $ao['V'])); 
         unset($ao['DA']);
      }
      $ao['failed'] = 'DA';
    break;
    case 'DB': // DB coordinate1, coordinate2        ; add an arc, from coordinate1 to coordinate2 (set center using V X=...)
      $ao['DB'] = $parts[1];
      if (isset($ao['V'])) {
         $ao['DP'] = array_merge($ao['DP'], DB_V($ao['DB'], $ao['V']));
         unset($ao['DB']);
      }

    break;
    case 'DC': // DC radius                          ; draw a circle (center taken from the previous V X=...  record, radius in nm
      $ao['DC'] = $parts[1];
      if (isset($ao['V'])) {
         $ao['DP'] = array_merge($ao['DP'], DC_V($ao['DC'], $ao['V']['X']));
         unset($ao['DC']);
      }
    break;
    case 'V' : // set center using V X=...

      $tmp = explode("=", $parts[1], 2);
      $ao['V'][$tmp[0]] = $tmp[1];
    break;
    case 'DY': // DY coordinate                      ; add a segment of an airway (NYI)
      $ao['failed'] = 'DY';
//      die('DY is not supported');
    break;
    default:
      if ((strlen($parts[0]) > 0) && (substr($parts[0], 0, 1) != '*')) {
        $ao['failed'] = 'unexpected';
//        die("unexpected input". $line."<br/>");
      }
    break;
  }

//  print_r($parts);

}

  return $all_ao;
}


function get_min_zoom($type) {
  return 1;
}
function get_max_zoom($type) {
  return 16;
}

$query = "START TRANSACTION";
pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);

$query = "TRUNCATE airspace";
pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);

$query = "TRUNCATE countries";
pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);

foreach ($files as $region => $file) {
  $fullname = "./data/$file";
  $soa = parseOpensoaring($fullname);
  $nicename = isset($files_nicenames[$region]) ?  $files_nicenames[$region] : basename($file, ".txt");

  foreach ($soa as $item) {
    foreach ($item['DP'] as $coord) {
      $lat = $coord['lat'];
      $lon = $coord['lon'];
    }
  }

  $import = array(
    'short_name' => $region,
    'name' => $nicename,
    'lat_min' => $lat,
    'lon_min' => $lon,
    'lat_max' => $lat,
    'lon_max' => $lon,
  );
  

  foreach ($soa as $item) {
    foreach ($item['DP'] as $coord) {
      $import['lat_max'] = max($coord['lat']+0.1, $import['lat_max']);
      $import['lon_max'] = max($coord['lon']+0.1, $import['lon_max']);
      $import['lat_min'] = min($coord['lat']-0.1, $import['lat_min']);
      $import['lon_min'] = min($coord['lon']-0.1, $import['lon_min']);
    }
  }

  foreach ($soa as $item) {
    if (!isset($item['AN'])) {
      continue;
    }
    $coords = json_encode($item['DP']);
    $min_lat = $min_lon = 360;
    $max_lat = $max_lon = -360;
    foreach ($item['DP'] as $c) {
      $min_lat = min($min_lat, $c['lat']);
      $min_lon = min($min_lon, $c['lon']);
      $max_lat = max($max_lat, $c['lat']);
      $max_lon = max($max_lon, $c['lon']);
    }
//    unset($item['DP']);
//echo "<pre>".print_r($item, true)."</pre>";

  $pts = json_decode($coords, true);
  $str = "";
  foreach ($pts as $pt) {
    $str .= $pt['lon']. ' '. $pt['lat'].', ';
  }

  $first = $pts[0];
  $last = $pts[count($pts)-1];
  if ($first['lat'] != $last['lat'] || $first['lon'] != $last['lon']) {
    $str .= $first['lon']. ' '.$first['lat'].', ';
  }

  $str = substr($str, 0, -2);

    $airspaceType = getAirspaceType($item['AN'], $item['AC']);

    $query = sprintf("INSERT INTO airspace (name, type, class, region, description, alt_min, alt_max, geometry) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', ST_ForceRHR(ST_GeomFromText('POLYGON((%s))', 4326)));",
      pg_escape_string($item['AN']),
      pg_escape_string($airspaceType),
      pg_escape_string($item['AC']),
      pg_escape_string($region),
      '',
      pg_escape_string($item['AL']),
      pg_escape_string($item['AH']),
      pg_escape_string($str)
    );
    pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);
   echo $item['AN']."<br/>\n";
  }

  $query = sprintf("INSERT INTO countries (short_name, name, lat_min, lon_min, lat_max, lon_max) VALUES ('%s', '%s', '%f', '%f', '%f', '%f')",
    pg_escape_string($import['short_name']),
    pg_escape_string($import['name']),
    $import['lat_min'],
    $import['lon_min'],
    $import['lat_max'],
    $import['lon_max']
  );
  echo "<pre>". print_r($import, true)."</pre>";

    pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);
}

$query = "COMMIT;";
pg_query($query) or die('Query failed:  <br/> <br/>' . pg_last_error(). ' <br/> <br/>' . $query);


?>